/* $RCSFile: CrmMailTemplateEditor.ascx.cs $
 * Apache license v2 2011 by Christian Surieux
 * $Source: ?/AddonNice.EMailing/PageModules/CRMEMailing/CrmMailTemplateEditor.ascx.cs $
 * $log$
 * Revision 23 2010/08/09 11:12:45  christian.surieux
 *   Added Log comment to all .ascx .aspx .master files
 * Revision 10 2010/08/09 10:13:18  christian.surieux
 *   Added Log comment to all cs files
 */

using System;
using System.Data;
using System.Drawing;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Xsl;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Diagnostics;
using System.Data.SqlClient;
using System.Web.Services.Protocols;
using System.Globalization;

using AddonNice.UI;
using AddonNice.Design;
using AddonNice.UI.WebControls;
using AddonNice.Security;
using AddonNice.UI.DataTypes;
using AddonNice.Configuration;
using AddonNice.Configuration.Settings;
using AddonNice.Diagnostics;
using AddonNice.Helpers;
using AddonNice.Settings;

using AddonNice.CRMWrapper;
using AddonNice.CRMEMailing;
using AddonNice.CRMEMailing.Settings.UI.WebControls;

using Telerik.Web.UI;

using Trace2    =   System.Diagnostics.Trace;
using System.Text.RegularExpressions;

namespace AddonNice.PageModules.CRMEMailing
{
 	#region Local TraceSwitch 
	
    public class CrmTemplateEditorTrace : ADNTraceSwitch
	{
		static public ADNTraceSwitch	Sw	            =	new ADNTraceSwitch("CrmTemplateEditorTrace","Special Trace Switch for CrmTemplateEditor");
	}
	
    #endregion Local TraceSwitch 
    
    /// <summary>
    /// Initializer class for CrmMailTemplateEditor 
    /// </summary>
    public class CrmMailTemplateEditorSI : PortalCRMModuleControlSI
    {
        public CrmMailTemplateEditorSI(ModuleSettingsDictionary bSettings,ISettingsOwner pm) : base()
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditorSI Ctor BEG.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
            InitSettings(bSettings,pm);
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditorSI Ctor END.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
        }
 
        public override void InitSettings(ModuleSettingsDictionary bSettings,ISettingsOwner pm)
        {             
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditorSI InitSettings BEG.",CrmTemplateEditorTrace.Sw.Info);
            
            base.InitSettings(bSettings,pm);

			// Specific parameters
            const string extgrp                     =   "CRMTEMPLATEEDITOR";
            

            SettingItemSpecial InputOnly            =   new SettingItemSpecial(AStr.InputOnly,new BooleanDataType());
            InputOnly.Required                      =   false;
            InputOnly.ResourceFile                  =   AStr.ResourceFile;
            InputOnly.ResourceItem                  =   AStr.ResourceItem;
            InputOnly.Value                         =   StStr.strFalse;
            InputOnly.Order                         =   10;
            InputOnly.ExtendedGroupName             =   extgrp;
            InputOnly.EnglishName                   =   "Input only mode";
            InputOnly.Description                   =   "When checked module will be in input only mode";
            bSettings.AddSetting(InputOnly);

            SettingItemSpecial HtmlMode             =   new SettingItemSpecial(AStr.HtmlMode,new BooleanDataType());
            HtmlMode.Required                       =   false;
            HtmlMode.ResourceFile                   =   AStr.ResourceFile;
            HtmlMode.ResourceItem                   =   AStr.ResourceItem;
            HtmlMode.Value                          =   StStr.strFalse;
            HtmlMode.Order                          =   20;
            HtmlMode.ExtendedGroupName              =   extgrp;
            HtmlMode.EnglishName                    =   "Allow Html mode";
            HtmlMode.Description                    =   "When checked editot will allow changing to html mode";
            bSettings.AddSetting(HtmlMode);

            SettingItemSpecial PreviewMode          =   new SettingItemSpecial(AStr.PreviewMode,new BooleanDataType());
            PreviewMode.Required                    =   false;
            PreviewMode.ResourceFile                =   AStr.ResourceFile;
            PreviewMode.ResourceItem                =   AStr.ResourceItem;
            PreviewMode.Value                       =   StStr.strFalse;
            PreviewMode.Order                       =   30;
            PreviewMode.ExtendedGroupName           =   extgrp;
            PreviewMode.EnglishName                 =   "Allow Html mode";
            PreviewMode.Description                 =   "When checked editot will allow changing to html mode";
            bSettings.AddSetting(PreviewMode);

            SettingItemSpecial CrmGoogleTranslate   =   new SettingItemSpecial(AStr.GoogleTranslate,new BooleanDataType());
			CrmGoogleTranslate.Order			    =	15;
            CrmGoogleTranslate.ResourceFile         =   AStr.ResourceFile;
            CrmGoogleTranslate.ResourceItem         =   AStr.ResourceItem;
            CrmGoogleTranslate.ExtendedGroupName    =   extgrp;
            CrmGoogleTranslate.Value			    =	StStr.strFalse;
			CrmGoogleTranslate.Description		    =	"Uncheck this setting if your client has no internet access.";
            CrmGoogleTranslate.EnglishName		    =	"Use Google Translation";
			bSettings.AddSetting(CrmGoogleTranslate);

            SettingItemSpecial Assign2Owner         =   new SettingItemSpecial(AStr.Assign2Owner,new BooleanDataType());
            Assign2Owner.Required                   =   false;
            Assign2Owner.ResourceFile               =   AStr.ResourceFile;
            Assign2Owner.ResourceItem               =   AStr.ResourceItem;
            Assign2Owner.Value                      =   StStr.strFalse;
            Assign2Owner.Order                      =   40;
            Assign2Owner.ExtendedGroupName          =   extgrp;
            Assign2Owner.EnglishName                =   "Assign to Owner";
            Assign2Owner.Description                =   "When checked, the new crm templates will be automatically assigned to selected owner";
            bSettings.AddSetting(Assign2Owner);

            WSListDataType defAcc                   =   new WSListDataType(CrmStr.systemuser,"name", "ID");
            SettingItemSpecial ModuleOwner          =   new SettingItemSpecial(AStr.TemplateOwner,defAcc);
            // default == CRMDefaultAccount or first
            ADNDynSystemUser wsys                   =   null;
            if (!string.IsNullOrEmpty(Config.CRMDefaultAccount))
            {
                string domainName                   =   string.Format("{0}\\{1}",Config.CRMDefaultAccountDomain,Config.CRMDefaultAccount);
                wsys                                =   Wrapper.Util.FindSystemUserByDomainName(domainName);
            }
            else 
            {
                wsys                                =   DynEntity.GetFromCache<ADNDynSystemUser>(CrmStr.systemuser,Wrapper,new ResetWrapperHandler(ResetWrapper),Wrapper.adminUserId);
            }
            if (wsys != null)
            {
                ModuleOwner.Value                   =   wsys.systemuserid.ToString();
                defAcc.DisplayTextValue             =   wsys.fullname;
            }
            ModuleOwner.Order                       =   50;
            ModuleOwner.ExtendedGroupName           =   extgrp;
            ModuleOwner.ResourceFile                =   AStr.ResourceFile;
            ModuleOwner.ResourceItem                =   AStr.ResourceItem;
            ModuleOwner.EnglishName                 =   "Owner";
            ModuleOwner.Description                 =   "Set a default owner for templates created with this module";
            ModuleOwner.SettingItemsEvent           +=  new SettingItemsEventHandler(ModuleOwner_SettingItemsEvent);
            bSettings.AddSetting(ModuleOwner);

            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditorSI InitSettings END.",CrmTemplateEditorTrace.Sw.Info);
        }
 
        /// <summary>
        /// Used to set the good display value in ModuleOwner when loaded from DB
        /// </summary>
        void ModuleOwner_SettingItemsEvent(object sender, SettingItemsEventArgs e)
        {
            if ( e.EventType != SettingItemsEventType.Loaded )
                return;
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditorSI ModuleOwner_SettingItemsEvent Guid: {0}", e.Data),CrmTemplateEditorTrace.Sw.Info);
            Guid custId     =   new Guid((string)e.Data);
            if ( custId == Guid.Empty )
                return;
            ADNDynSystemUser dynac  =   DynEntity.GetFromCache<ADNDynSystemUser>(CrmStr.systemuser,Wrapper,new ResetWrapperHandler(ResetWrapper),custId);
            WSListDataType defAcc   =   (WSListDataType )((SettingItem)sender).DataType;
            defAcc.DisplayTextValue =   dynac.fullname;
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditorSI ModuleOwner_SettingItemsEvent END name: {0}", dynac.name),CrmTemplateEditorTrace.Sw.Info);
        }
    }

    /// <summary>
    ///	Dedicated input (only) module for leads
    /// </summary>
    public partial class CrmMailTemplateEditor : PortalEmailingModule
    {
        #region General implementation
        
        public override Guid GuidID
        {
            get
            {
                return new Guid(AStr.CrmMailTemplateEditorGuid);
            }
        }
        #endregion General implementation

        bool CreateMode
        {
            get
            {
                return Settings[AStr.InputOnly].ToBool() || ( new Guid(RadCbTemplates.SelectedValue) == Guid.Empty ) || ChkCreateCopy.Checked ;
            }
        }

        bool GoogleTranslate
        {
            get
            {
                return Settings[AStr.GoogleTranslate].ToBool()&& portalSettings.CustomSettings[StStr.SiteSettingsGoogleTranslate].ToBool();
            }
        }

        public CrmMailTemplateEditor()
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor Ctor Called.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
            // Validators : name Ok
            ValidationGroup                     =   "Main";
        }

        public override void InitPortalModuleControl(ModuleSettingsDictionary bSettings)
        {
            PortalModuleInitializer             =   new CrmMailTemplateEditorSI(bSettings,this);
        }

    
        ArrayList _AllEntities =   null;
        ArrayList GetAllEntities(bool useAll)
        {
            if (_AllEntities == null)
            {
                _AllEntities          =   MetadataCache.GetAllEntities(Wrapper,useAll);
            }
            return _AllEntities;
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor Page_Load  BEG IsPostBack: {0}, IsInAsyncPostBack: {1}.",IsPostBack,IsInAsyncPostBack),
                    (ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
            if (!IsPostBack)
            {
                BodyEditor.EnsureToolsFileLoaded();
                int idx                         =   0;
                if ( GoogleTranslate )
                {
                    BodyEditor.OnClientCommandExecuting = "OnClientCommandExecuting";
                    // Add and fill the translate editor tool 
                    EditorDropDown ddn          =   new EditorDropDown("TranslateTool");
                    ddn.Text                    =   LocalizeHelper.Localize("EDITOR_TRANSLATE","Translate");
                    ddn.Width = new Unit("130px");
                    BodyEditor.Tools[0].Tools.Insert(idx++,ddn);
                    string[] SupportedLanguages =   PortalSettings.SpellCheckerCultures;
                    foreach ( string dicLang in SupportedLanguages )
                    {
                        ddn.Items.Add(new EditorDropDownItem(new CultureInfo(dicLang).DisplayName,dicLang.Substring(0,2)));
                    }
                }
                // the macro combo tool
                EditorDropDown ddInsert         =   new EditorDropDown("InsertTool");
                ddInsert.Text                   =   LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,"TEMPLATE_INSERTMACRO","Insert macro");
                ddInsert.Width                  =   new Unit("130px");
                BodyEditor.Tools[0].Tools.Insert(idx,ddInsert);

                // Fill Types
                RadCbTypes.DataTextField    =   "displayName";
                RadCbTypes.DataValueField   =   "entityName";
                RadCbTypes.DataSource       =   GetAllEntities(true);
                RadCbTypes.DataBind();
                RadCbTypes.Sort             =    RadComboBoxSort.Ascending;
                RadCbTypes.SortItems();

                string str                  =   LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,"TEMPLATE_GLOBAL","Global");
                RadComboBoxItem it          =   new RadComboBoxItem(str,"global");
                it.Selected                 =   true;
                RadCbTypes.Items.Insert(0,it);

                // Fill languages : Get it from Crm ?
                str                         =   LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,"TEMPLATE_ENGLISH","English");
                it                          =   new RadComboBoxItem(str,"1033");
                RadCbLanguage.Items.Add(it);
                str                         =   LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,"TEMPLATE_FRENCH","French");
                it                          =   new RadComboBoxItem(str,"1036");
                RadCbLanguage.Items.Add(it);

                // Fill templates
                InitTemplates();
                ChkCreateCopy.Checked       =   false;
                ChkCreateCopy.Enabled       =   false;

                // subject editor
                EditorToolGroup main            =   new EditorToolGroup();
                EditSubject.Tools.Add(main);

                EditorTool body                 =   new EditorTool();
                body.Name                       =   "Bold";
                body.ShortCut                   =   "CTRL+B";
                main.Tools.Add(body);

                EditorTool italic               =   new EditorTool();
                italic.Name                     =   "Italic";
                italic.ShortCut                 =   "CTRL+I";
                main.Tools.Add(italic);

                EditorTool underline            =   new EditorTool();
                underline.Name                  =   "Underline";
                underline.ShortCut              =   "CTRL+U";
                main.Tools.Add(underline); 

                if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor Page_Load Count: {0}.",RadCbTemplates.Items.Count),CrmTemplateEditorTrace.Sw.Info);
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor Page_Load  END.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
        }


        void InitTemplates()
        {
            RadCbTemplates.Items.Clear();
            if ( !Settings[AStr.InputOnly].ToBool() )
            {
                SortedList<Guid,ADNDynTemplate> Lst =   ADNDynTemplate.ReadAll(Wrapper,ResetWrapper,string.Empty,-1,false);
                foreach ( Guid TemplateID in Lst.Keys )
                {
                    ADNDynTemplate dynTemp          =   Lst[TemplateID];
                    RadComboBoxItem it              = new RadComboBoxItem(dynTemp.title,TemplateID.ToString());
                    RadCbTemplates.Items.Add(it);
                }
                RadCbTemplates.Sort                 =    RadComboBoxSort.Ascending;
                RadCbTemplates.SortItems();
            }
            string str                              =   LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,"TEMPLATE_NEW","New");
            RadComboBoxItem it0                     =   new RadComboBoxItem(str,Guid.Empty.ToString());
            RadCbTemplates.Items.Insert(0,it0);
        }

        /*
<?xml version="1.0" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text" indent="no"/>
<xsl:template match="/data">
<![CDATA[<P>Dear ]]>
<xsl:choose><xsl:when test="contact/fullname"><xsl:value-of select="contact/fullname" /></xsl:when><xsl:otherwise>Valued Customer</xsl:otherwise></xsl:choose>
<![CDATA[,</P> <P>We have not heard from you for a while. We wanted to check in and make sure that you are still having a great experience using our product(s). We have asked ]]>
<xsl:choose><xsl:when test="contact/ownerid/@name"><xsl:value-of select="contact/ownerid/@name" /></xsl:when><xsl:otherwise>a customer service representative</xsl:otherwise></xsl:choose>
<![CDATA[ to contact you next week to get your feedback on the product(s) you are currently using and to give you details about our upcoming products.</P> <P>Thank you.</P>]]>
</xsl:template></xsl:stylesheet>         
        
         
<template><text><![CDATA[Microsoft Dynamics CRM&nbsp;: ]]></text><slugs><slug><entity>asyncoperation</entity><attribute>name</attribute></slug><default>tche de suppression en bloc</default></slugs><text><![CDATA[ termine]]></text></template>
Microsoft Dynamics CRM&nbsp;: <span class="dataslug"><span style="display:none;" id="DataSlug">{!asyncoperation:name;tche de suppression en bloc}</span>{!:;tche de suppression en bloc}</span> termine
         * 
         
         */
        string xsltdoc="<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"><xsl:output method=\"text\" indent=\"no\"/><xsl:template match=\"template\"><xsl:for-each select=\"*\">"+
			"<xsl:apply-templates select=\".\" /></xsl:for-each></xsl:template><xsl:template match=\"text\"><xsl:value-of select=\".\" /></xsl:template>"+
            "<xsl:template match=\"slugs\"><![CDATA[{]]><xsl:for-each select=\"slug\"><xsl:value-of select=\"entity\" />/<xsl:value-of select=\"attribute\" />;</xsl:for-each><xsl:value-of select=\"default\" />"+
            "<![CDATA[}]]></xsl:template></xsl:stylesheet>";

        string xsltdoc0="<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"><xsl:output method=\"text\" indent=\"no\"/><xsl:template match=\"template\"><xsl:for-each select=\"*\">"+
			"<xsl:apply-templates select=\".\" /></xsl:for-each></xsl:template><xsl:template match=\"text\"><xsl:value-of select=\".\" /></xsl:template>"+
            "<xsl:template match=\"slugs\"><![CDATA[<span class=\"dataslug\"><span style=\"display:none;\" id=\"DataSlug\">{!]]><xsl:for-each select=\"slug\"><xsl:value-of select=\"entity\" />:<xsl:value-of select=\"attribute\" />;</xsl:for-each><xsl:value-of select=\"default\" />"+
            "<![CDATA[}</span>{!]]><xsl:for-each select=\"slug\"><xsl:value-of select=\"entity/@displayname\" />:<xsl:value-of select=\"attribute/@displayname\" />;</xsl:for-each><xsl:value-of select=\"default\" /><![CDATA[}</span>]]></xsl:template></xsl:stylesheet>";

         
        string TransformXmlSubject(string xmlbody)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor TransformXmlSubject BEG body: [{0}].",xmlbody),CrmTemplateEditorTrace.Sw.Info);
            string result                           =   string.Empty;
            try
            {
                XmlDocument doc				        =   new XmlDocument();
			    doc.LoadXml(xmlbody);

			    XslCompiledTransform xslt			=   new XslCompiledTransform();

			    // build parameter list to pass to stylesheet
			    XsltArgumentList xslArg		        =   new XsltArgumentList();
                XmlDocument xdoc                    =   new XmlDocument();
                xdoc.LoadXml(xsltdoc);
			    //Load the stylesheet
			    xslt.Load(xdoc,null,null);
			    //Transform the data
                StringBuilder sb                    =   new StringBuilder();
                using (StringWriter sw              =   new StringWriter(sb))
                {
                    xslt.Transform(doc.CreateNavigator(), xslArg,sw);
                }
                result                              =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor TransformXmlSubject ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor TransformXmlSubject END: [{0}].",result),CrmTemplateEditorTrace.Sw.Info);
			return result;
        }

        string TransformXml(string xmlbody,string dummyObj)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor TransformXml BEG body: [{0}].",xmlbody),CrmTemplateEditorTrace.Sw.Info);
            string result                           =   string.Empty;
            try
            {
                XmlDocument doc				        =   new XmlDocument();
			    doc.LoadXml(dummyObj);

			    XslCompiledTransform xslt			=   new XslCompiledTransform();

			    // build parameter list to pass to stylesheet
			    XsltArgumentList xslArg		        =   new XsltArgumentList();
                XmlDocument xdoc                    =   new XmlDocument();
                xdoc.LoadXml(xmlbody);
			    //Load the stylesheet
			    xslt.Load(xdoc,null,null);
			    //Transform the data
                StringBuilder sb                    =   new StringBuilder();
                using (StringWriter sw              =   new StringWriter(sb))
                {
                    xslt.Transform(doc.CreateNavigator(), xslArg,sw);
                }
                result                              =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor TransformXml ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor TransformXml END: [{0}].",result),CrmTemplateEditorTrace.Sw.Info);
			return result;
        }

        /// <summary>
        /// Generate an xml template to allow applying the xsl template to obtain an editable text containing special markers
        /// </summary>
        string GenerateEntityXml(string xmlbody,string entityNm,EditorDropDown attribCombo)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor GenerateEntityXml BEG entityName: {0}, body: [{1}].",entityNm,xmlbody),CrmTemplateEditorTrace.Sw.Info);
            string result                       =   string.Empty;
            try
            {
                EntityConfig conf               =   new EntityConfig(Wrapper,entityNm,EntityInternalItems.IncludeAttributes,true);
                StringBuilder sb                =   new StringBuilder();
                sb.AppendFormat("<?xml version=\"1.0\" encoding=\"utf-8\"?><data><{0}>",entityNm);
                foreach(string att in conf.attributesDic.Keys)
                {
                    string searchstr            =   string.Format("{0}/{1}",entityNm,att);
                    // special attributes like owner and lookup
                    bool addAttribute           =   false;
                    string displayName          =   conf.AttFieldAsString(att,InternalAttributeFields.DisplayName);
                    AttributeInternalType tp    =   (AttributeInternalType)conf.AttFieldAsInt(att,InternalAttributeFields.AttributeType);
                    if ( (tp & AttributeInternalType.Virtual) == AttributeInternalType.Virtual ) // skip all addendum
                        continue;
                    if ( (tp & AttributeInternalType.Owner) == AttributeInternalType.Owner )
                    {
                        addAttribute            =   true;
                        searchstr               =   searchstr+"/@name";
                    }
                    else if ( (tp & AttributeInternalType.Lookup) == AttributeInternalType.Lookup )
                    {
                        addAttribute            =   true;
                        searchstr               =   searchstr+"/@name";
                    }
                    string defaultvalue         =   string.Empty;
                    int idx                     =   xmlbody.IndexOf(searchstr);
                    if ( idx > -1 )
                    {
                        //aaa</xsl:when><xsl:otherwise>bbbb</xsl:otherwise>
                        string b2               =   xmlbody.Substring(idx);
                        string tag1             =   "</xsl:when><xsl:otherwise>";
                        idx                     =   b2.IndexOf(tag1); // 3
                        int len                 =   tag1.Length; //26
                        string tag2             =   "</xsl:otherwise>";
                        int idx2                =   b2.IndexOf(tag2); // 33
                        defaultvalue            =   b2.Substring(idx+len,idx2-idx-len);// 29,4
                    }
                    //<fullname>{contact/fullname;Valued Customer}</fullname>
                    // always add the default attribute in case we want to add the guid value
                    sb.AppendFormat("<{0}>{{{1};{2}}}</{0}>",att,searchstr,defaultvalue);
                    if (( attribCombo != null ) && !string.IsNullOrEmpty(displayName) )
                        attribCombo.Items.Add(new EditorDropDownItem(displayName,string.Format("{{{0}/{1};}}",entityNm,att)));
                    // need the special 'name' attribute
                    if ( addAttribute )
                    {
                        sb.AppendFormat("<{0} name=\"{{{1};{2}}}\"></{0}>",att,searchstr,defaultvalue);
                        if (( attribCombo != null )&& !string.IsNullOrEmpty(displayName) )
                            attribCombo.Items.Add(new EditorDropDownItem(string.Format("{0}/name",displayName),string.Format("{{{0}/{1}/@name;}}",entityNm,att)));
                    }
                }
                sb.AppendFormat("</{0}></data>",entityNm);
                result                          =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor GenerateEntityXml ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor GenerateEntityXml result: [{0}].",result),CrmTemplateEditorTrace.Sw.Info);
			return result;
        }

        /// <summary>
        /// Refresh display with values from selected template
        /// </summary>
        void RadCbTemplates_SelectedIndexChanged(object o,RadComboBoxSelectedIndexChangedEventArgs e)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RadCbTemplates_SelectedIndexChanged BEG e.value: [{0}].",e.Value),CrmTemplateEditorTrace.Sw.Info);
            try
            {
                Guid TemplateID         =   new Guid(RadCbTemplates.SelectedValue);
                // New selected
                if ( TemplateID == Guid.Empty )
                {
                    // Do not empty text to allow new input
                    RadCbTypes.Items.FindItemByValue("global").Selected =   true;
                    ChkCreateCopy.Checked   =   false;
                    ChkCreateCopy.Enabled   =   false;
                    return;
                }
                ChkCreateCopy.Enabled   =   true;
                ChkCreateCopy.Checked   =   false;
                ADNDynTemplate im       =   DynEntity.GetFromCache<ADNDynTemplate>(CrmStr.template,Wrapper,ResetWrapper,TemplateID);
                TbTitle.Text            =   im.title;
                RadComboBoxItem it      =   RadCbLanguage.FindItemByValue(im.languagecode.ToString());
                if ( it != null )
                    it.Selected         =   true;
                it                      =   RadCbTypes.FindItemByValue(im.templatetypecode);
                if ( it != null )
                    it.Selected         =   true;
                TbDescription.Text      =   im.description;
                EditorDropDown ddInsert =   (EditorDropDown)BodyEditor.Tools[0].Tools[(GoogleTranslate) ? 1:0];
                if ( ddInsert != null )
                    ddInsert.Items.Clear();
                string dummyObj         =   GenerateEntityXml(im.body,im.templatetypecode,ddInsert);
                if ( !string.IsNullOrEmpty(im.subjectpresentationxml) )
                     EditSubject.Content=   TransformXmlSubject(im.subjectpresentationxml);
                else if ( !string.IsNullOrEmpty(im.subject) )
                    EditSubject.Content=   TransformXmlSubject(im.subject);
                else EditSubject.Content=   string.Empty;
                if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RadCbTemplates_SelectedIndexChanged subjectpresentationxml: [{0}].",im.subjectpresentationxml),CrmTemplateEditorTrace.Sw.Info);
                if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RadCbTemplates_SelectedIndexChanged presentationxml: [{0}].",im.presentationxml),CrmTemplateEditorTrace.Sw.Info);
                BodyEditor.Content      =   TransformXml(im.body,dummyObj);
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor RadCbTemplates_SelectedIndexChanged ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine("[V]CrmMailTemplateEditor RadCbTemplates_SelectedIndexChanged END.",CrmTemplateEditorTrace.Sw.Info);
        }

        string ReFormatSubject(string content)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor ReFormatSubject BEG content: {0}",content),CrmTemplateEditorTrace.Sw.Info);
            string result           =   string.Empty;
            try
            {
                string start            =   "<template><text><![CDATA[";
                StringBuilder sb        =   new StringBuilder(start);
                sb.Append(RegExifyOutXml(content));
                sb.Append("]]></text></template>");
                result                  =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor ReFormatSubject ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor ReFormatSubject END result: {0}",result),CrmTemplateEditorTrace.Sw.Info);
            return result;
        }

        /// <summary>
        /// Takes the edited contents and format them for Crm templates
        /// </summary>
        string ReFormat(string content)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor ReFormat BEG content: {0}",content),CrmTemplateEditorTrace.Sw.Info);
            string result           =   string.Empty;
            try
            {
                string start            =   "<?xml version=\"1.0\" ?><xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\">"
                                            +"<xsl:output method=\"text\" indent=\"no\"/><xsl:template match=\"/data\"><![CDATA[";
                StringBuilder sb        =   new StringBuilder(start);
                sb.Append(RegExifyOut(content));
                sb.Append("]]></xsl:template></xsl:stylesheet>");
                result                  =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor ReFormat ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor ReFormat END result: {0}",result),CrmTemplateEditorTrace.Sw.Info);
            return result;
        }

        string RegExifyOut(string content)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOut BEG content: {0}",content),CrmTemplateEditorTrace.Sw.Info);
            string result           =   string.Empty;
            try
            {
                // <xsl:choose><xsl:when test="contact/ownerid/@name"><xsl:value-of select="contact/ownerid/@name" /></xsl:when><xsl:otherwise>a customer service representative</xsl:otherwise></xsl:choose><![CDATA[
                //string regexpat         =   @"{(?<m1>[a-zA-Z0-9_\-/@]+);(?<m2>.*)}";
                string regexpat         =   @"{(?<m1>[^;]+);(?<m2>[^}]*)}";
                Regex reg               =   new Regex(regexpat);
                MatchCollection mc      =   reg.Matches(content);
                StringBuilder sb        =   new StringBuilder();
                int startidx            =   0;
                string extracted        =   string.Empty;
                for(int i =0; i < mc.Count ; i++ )
                {
                    Match m             =   mc[i];
                    GroupCollection gc  =   m.Groups;
                    Group g1            =   gc["m1"];
                    string m1           =   g1.Value;
                    Group g2            =   gc["m2"];
                    string m2           =   g2.Value;
                    extracted           =   content.Substring(startidx,m.Index-startidx);
                    if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOut m1: {0}, m2: {1}, extracted: {2}",m1,m2,extracted),CrmTemplateEditorTrace.Sw.Info);
                    sb.Append(extracted);
                    startidx            =   (m.Index+m.Length);
                    sb.Append("]]>");
                    string append       =   string.Format("<xsl:choose><xsl:when test=\"{0}\"><xsl:value-of select=\"{0}\" /></xsl:when><xsl:otherwise>{1}</xsl:otherwise></xsl:choose><![CDATA[",m1,m2);
                    if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOut append: {0}",append),CrmTemplateEditorTrace.Sw.Info);
                    sb.Append(append);
                }
                // Append the text left
                extracted               =   content.Substring(startidx);
                sb.Append(extracted);
                result                  =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor RegExifyOut ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOut END result: {0}",result),CrmTemplateEditorTrace.Sw.Info);
            return result;
        }

        string RegExifyOutXml(string content)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOutXml BEG content: {0}",content),CrmTemplateEditorTrace.Sw.Info);
            string result           =   string.Empty;
            try
            {
                // <xsl:choose><xsl:when test="contact/ownerid/@name"><xsl:value-of select="contact/ownerid/@name" /></xsl:when><xsl:otherwise>a customer service representative</xsl:otherwise></xsl:choose><![CDATA[
                //string regexpat         =   @"{(?<m1>[a-zA-Z0-9_\-/@]+);(?<m2>.*)}";
                string regexpat         =   @"{(?<m1>[^/]+)/(?<m2>[^;]+);(?<m3>[^}]*)}";
                Regex reg               =   new Regex(regexpat);
                MatchCollection mc      =   reg.Matches(content);
                StringBuilder sb        =   new StringBuilder();
                int startidx            =   0;
                string extracted        =   string.Empty;
                for(int i =0; i < mc.Count ; i++ )
                {
                    Match m             =   mc[i];
                    GroupCollection gc  =   m.Groups;
                    Group g1            =   gc["m1"];
                    string m1           =   g1.Value;
                    Group g2            =   gc["m2"];
                    string m2           =   g2.Value;
                    extracted           =   content.Substring(startidx,m.Index-startidx);
                    Group g3            =   gc["m3"];
                    string m3           =   g3.Value;
                    if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOutXml m1: {0}, m2: {1}, m3: {2}, extracted: {3}",new object[]{m1,m2,m3,extracted}),CrmTemplateEditorTrace.Sw.Info);
                    sb.Append(extracted);
                    startidx            =   (m.Index+m.Length);
                    sb.Append("]]></text>");
                    string append       =   string.Format("<slugs><slug><entity>{0}</entity><attribute>{1}</attribute></slug><default>{2}</default></slugs><text><![CDATA[",m1,m2,m3);
                    if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOutXml append: {0}",append),CrmTemplateEditorTrace.Sw.Info);
                    sb.Append(append);
                }
                // Append the text left
                extracted               =   content.Substring(startidx);
                sb.Append(extracted);
                result                  =   sb.ToString();
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor RegExifyOutXml ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor RegExifyOutXml END result: {0}",result),CrmTemplateEditorTrace.Sw.Info);
            return result;
        }

        /// <summary>
        /// Create or update the temlate
        /// </summary>
        void CrmMailTemplateEditor_ADNUpdateModule(object sender, ModuleEventArgs e)
        {
            if (CrmTemplateEditorTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor CrmMailTemplateEditor_ADNUpdateModule BEG CreateMode: {0}.", CreateMode),CrmTemplateEditorTrace.Sw.Info);
            try
            {
                ADNDynTemplate im           =   null;
                if ( CreateMode )
                {
                    im                      =   new ADNDynTemplate(ResetWrapper,TbTitle.Text.Trim());
                    if ( ChkCreateCopy.Checked )
                    {
                        Guid TemplateID         =   new Guid(RadCbTemplates.SelectedValue);
                        ADNDynTemplate orgTemp  =   DynEntity.GetFromCache<ADNDynTemplate>(CrmStr.template,Wrapper,ResetWrapper,TemplateID);
                        bool ownerChanged       =   false;
                        orgTemp.CopyTo(im,false,ref ownerChanged);
                    }
                    im.SetOwnerFromUserId();

                }
                else
                {
                    Guid TemplateID         =   new Guid(RadCbTemplates.SelectedValue);
                    im                      =   DynEntity.GetFromCache<ADNDynTemplate>(CrmStr.template,Wrapper,ResetWrapper,TemplateID);
                    if ( Settings[AStr.Assign2Owner].ToBool() )
                        im.ownerid		    =   Settings[AStr.TemplateOwner].ToGuid();
                }
                im.title                    =   TbTitle.Text.Trim();
                im.subject                  =   ReFormat(EditSubject.Content);
                im.subjectpresentationxml   =   ReFormatSubject(EditSubject.Content);
                im.languagecode	            =   int.Parse(RadCbLanguage.SelectedValue);
                if ( RadCbTypes.SelectedValue != "global" )
                    im.templatetypecode     =   RadCbTypes.SelectedValue;
                im.description              =   TbDescription.Text.Trim();
                im.body                     =   ReFormat(BodyEditor.Content);
                im.presentationxml          =   ReFormatSubject(BodyEditor.Content);
                Guid EntID                  =   Guid.Empty;
                if ( CreateMode )
                {
                    EntID                   =   im.Create();
                    if (CrmTemplateEditorTrace.Sw.TraceVerbose)
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor CrmMailTemplateEditor_ADNUpdateModule END Created EntityID: {0}, title: {1}", EntID, im.title),CrmTemplateEditorTrace.Sw.Info);
                }
                else
                {
                    EntID                   =   im.templateid;
                    im.Update();
                    if (CrmTemplateEditorTrace.Sw.TraceVerbose)
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor CrmMailTemplateEditor_ADNUpdateModule END updadted templateid: {0}, title: {1}",im.templateid, im.title),CrmTemplateEditorTrace.Sw.Info);
                }
                if ( CreateMode )
                    InitTemplates();
                else
                {
                    RadComboBoxItem it      =   RadCbTemplates.Items.FindItemByValue(EntID.ToString());
                    if ( it != null )
                        it.Text             =   im.title;
                }
            }
            catch (Exception ex)
            {
                if (CrmTemplateEditorTrace.Sw.TraceError)
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CrmMailTemplateEditor CrmMailTemplateEditor_ADNUpdateModule  ex: {0}", ex),CrmTemplateEditorTrace.Sw.Info);
                throw;
            }
        }

        /// <summary>
        /// Reset the listbox to values from the fields grid
        /// </summary>
        void ResetBtn_Click(object sender, ImageClickEventArgs e)
        {
            TbTitle.Text            =   string.Empty;
            TbDescription.Text      =   string.Empty;
            EditSubject.Content     =   string.Empty;
            BodyEditor.Content      =   string.Empty;
        }

         protected override void OnPreRender(EventArgs e)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CrmMailTemplateEditor OnPreRender BEG CRM GoogleTranslate: {0}, IsPostBack: {1}, IsInAsyncPostBack: {2}",GoogleTranslate,IsPostBack,IsInAsyncPostBack),CrmTemplateEditorTrace.Sw.Info);
            if ( GoogleTranslate && !IsInAsyncPostBack )
            {
                string gPath    =   "http://www.google.com/jsapi";
                ((ADNPage)Page).RegisterADNClientScriptInclude("GoggleAPI", gPath);
                gPath           =   AddonNice.Settings.Path.ToAbsolute(Config.PortalScriptsPath + "/GoggleTranslator.js");
                ((ADNPage)Page).RegisterADNClientScriptInclude("GoggleTranslator", gPath);
            }
            base.OnPreRender(e);
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor OnPreRender END.",CrmTemplateEditorTrace.Sw.Info);
        }

       #region Std overrides

        public override void Page_PreInit(object sender, EventArgs e)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor Page_PreInit BEG.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
            base.Page_PreInit(sender, e);
            updateBtn.SkinID = "Update_" + Helpers.LocalizeHelper.GetCurrentNeutralCultureName();
            ResetBtn.SkinID = "Reset_" + Helpers.LocalizeHelper.GetCurrentNeutralCultureName();
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor Page_PreInit  END.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
        }

        override protected void OnInit(EventArgs e)
        {
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor OnInit BEG.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
            InitializeComponent();
            base.OnInit(e);
            if ( CrmTemplateEditorTrace.Sw.TraceVerbose || ModuleTraceSwitch.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]CrmMailTemplateEditor OnInit  END.",(ModuleTraceSwitch.Sw.TraceVerbose) ? ModuleTraceSwitch.Sw.Info:CrmTemplateEditorTrace.Sw.Info);
        }

        private void InitializeComponent()
        {
            Load                                +=  new System.EventHandler(Page_Load);
            ADNUpdateModule                     +=  new PortalModuleEventHandler(CrmMailTemplateEditor_ADNUpdateModule);
            ResetBtn.Click                      +=  new ImageClickEventHandler(ResetBtn_Click);
            RadCbTemplates.AutoPostBack         =   true;
            RadCbTemplates.SelectedIndexChanged +=  new Telerik.Web.UI.RadComboBoxSelectedIndexChangedEventHandler(RadCbTemplates_SelectedIndexChanged);
             // Init Editor
            BodyEditor.AutoResizeHeight             =   false;
            string[] SupportedLanguages              =   PortalSettings.SpellCheckerCultures;
            foreach ( string dicLang in SupportedLanguages )
                BodyEditor.Languages.Add(new SpellCheckerLanguage(dicLang,new CultureInfo(dicLang).DisplayName));
		    // Fixe allowed edit modes
            BodyEditor.EditModes                    =    EditModes.Design;
            if ( Settings[AStr.HtmlMode].ToBool() )
                BodyEditor.EditModes                =    EditModes.Design|EditModes.Html;
            if ( Settings[AStr.PreviewMode].ToBool() )
                BodyEditor.EditModes                |=   EditModes.Preview;

            string moduleImageFolder                =   portalSettings.CustomSettings[StStr.SiteSettingsDefaultImageFolder].ToString();
            string fullImagePath                    =   portalSettings.DefaultImageVirtualPath;

            string[] folderPath                     =   new string[] { fullImagePath };
            BodyEditor.ImageManager.ViewPaths       =   folderPath;
            BodyEditor.ImageManager.UploadPaths     =   folderPath;
            BodyEditor.ImageManager.DeletePaths     =   folderPath;

            folderPath                              =   new string[] { portalSettings.DefaultMediaVirtualPath };
            BodyEditor.MediaManager.ViewPaths       =   folderPath;
            BodyEditor.MediaManager.UploadPaths     =   folderPath;
            BodyEditor.MediaManager.DeletePaths     =   folderPath;

            folderPath                              =   new string[] { portalSettings.DefaultFlashVirtualPath };
            BodyEditor.FlashManager.ViewPaths       =   folderPath;
            BodyEditor.FlashManager.UploadPaths     =   folderPath;
            BodyEditor.FlashManager.DeletePaths     =   folderPath;

            folderPath                              =   new string[] { portalSettings.DefaultSilverlightVirtualPath };
            BodyEditor.SilverlightManager.ViewPaths   =   folderPath;
            BodyEditor.SilverlightManager.UploadPaths =   folderPath;
            BodyEditor.SilverlightManager.DeletePaths =   folderPath;

            folderPath                              =   new string[] { portalSettings.DefaultDocumentVirtualPath };
            BodyEditor.DocumentManager.ViewPaths    =   folderPath;
            BodyEditor.DocumentManager.UploadPaths  =   folderPath;
            BodyEditor.DocumentManager.DeletePaths  =   folderPath;

            folderPath                              =   new string[] { portalSettings.DefaultTemplateVirtualPath };
            BodyEditor.TemplateManager.ViewPaths    =   folderPath;
            BodyEditor.TemplateManager.UploadPaths  =   folderPath;
            BodyEditor.TemplateManager.DeletePaths  =   folderPath;
       }

        #endregion Std overrides
    }
}