﻿/* $RCSFile: CRMEMailingInit.cs $
 * Apache license v2 2011 by Christian Surieux
 * $Source: ?/AddonNice.Emailing.Settings/CRMEMailingInit.cs $
 * $log$
 * Revision 10 2010/08/09 10:13:18  christian.surieux
 *   Added Log comment to all cs files
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;

using AddonNice;
using AddonNice.UI.DataTypes;
using AddonNice.Configuration;
using AddonNice.Security;
using AddonNice.Diagnostics;
using AddonNice.Settings;
using AddonNice.Country;
using AddonNice.Helpers;

using AddonNice.CRMWrapper;

using Trace2    =   System.Diagnostics.Trace;
using System.Data.SqlClient;

namespace AddonNice.CRMEMailing
{

    #region TraceSwitch for CRMEMailing
	public class CRMEMailingTrace
    {
		static public ADNTraceSwitch	Sw	=	new ADNTraceSwitch("CRMEMailingTrace","Trace Switch for CRMEMailingTrace package");
	}
    
	#endregion TraceSwitch Crm Image

    #region Install class for Package
	
    public class InstallPackage : IPackageInstaller
	{
        /// <summary>
        /// Recover or create the IFrame Page in this portal
        /// </summary>
        static public int GetIframePageId(int PortalID)
        {
            PortalsDB PDB           =   new PortalsDB();
            PagesDB pagesDB	        =	new PagesDB();
            
            bool alreadyMoved       =   false;
            string result    =   null;
            try
            {
                result              =   PDB.GetPortalSingleCustomSettings(PortalID,AStr.IFramePageMenu);
                if ( !string.IsNullOrEmpty(result) )
                    alreadyMoved    =   true;
            }
            catch ( Exception exp )
            {
                if ( InstallTraceSwitch.Sw.TraceError )
                    Trace.WriteLine(string.Format(CultureInfo.InvariantCulture,"[E]InstallPackage GetIframePageId GetPortalSingleCustomSettings ex: {0}",exp),InstallTraceSwitch.Sw.Info);
            }
            if ( alreadyMoved )
                return int.Parse(result);

            string DBAllUser        =   Config.InternallAllUsersRole;
            // Create an Iframe Page in this portal
            int IFramePageId            =   pagesDB.AddPage(PortalID, StStr.PageEMailingMenuID,"IFrame Pages",StStr.strDefaultSkin, 30,
                                                        3,0,0,0,0,0,DBAllUser);
            // Save this pageid in portalsettings 
	        PDB.UpdatePortalSetting(PortalID,AStr.IFramePageMenu,IFramePageId.ToString());  
            /*
            // Create a new page for the Iframe
            int wPageId                 =   pagesDB.AddPage(PortalID, IFramePageId,"Fill download session documents",StStr.strDefaultSkin, 10,
                                                        3,0,0,0,0,0,DBAllUser);
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_EN","Fill download session documents.");
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_FR","Insérer des documents dans une session téléchargement.");
             * */
            return IFramePageId;
        }

        /// <summary>
        /// Move EMailing CRM Menu Page to common portal without parent page 
        /// </summary>
        public static bool RecoverMainEMailingMenu(CRMWrapperBase wrapper,int portalID)
        {

            PagesDB pagesDB	        =	new PagesDB();
            ModulesDB modDB			=	new ModulesDB();

            string DBAllUser        =   Config.InternallAllUsersRole;
            string DBAllUser2		=   Config.InternallAllUsersRole + ";";

            string DBAdmin          =   Config.InternalAdminRole;
            string DBAdmin2         =   Config.InternalAdminRole + ";";

            

            // Test common page portal
            bool alreadyMoved       =   false;
            SqlDataReader result    =   null;
            try
            {
                result = pagesDB.GetPortalPage(StStr.PageEMailingMenuID);
                result.Read();
                if ( result[StStr.DBPortalID] != DBNull.Value )
                    alreadyMoved = ( StStr.CommonPortalID == (int)result[StStr.DBPortalID] );
            }
            catch ( Exception exp )
            {
                if ( InstallTraceSwitch.Sw.TraceError )
                    Trace.WriteLine(string.Format(CultureInfo.InvariantCulture,"[E]InstallPackage Install GetPortalPage ex: {0}",exp),InstallTraceSwitch.Sw.Info);
            }
            finally
            {
                if ( result != null )
                    result.Close();
            }
            Guid moduleGuid             =   Guid.Empty;
            int moduleDefId             =   0;
            if ( !alreadyMoved )
            {
                pagesDB.RecoverPage(StStr.PageEMailingMenuID,StStr.CommonPortalID,0,0);
                pagesDB.UpdatePageSettings(StStr.PageEMailingMenuID,"PAGEMETADESCRIPTION_EN","Main menu for CRM Emailing package.");
                pagesDB.UpdatePageSettings(StStr.PageEMailingMenuID,"PAGEMETADESCRIPTION_FR","Menu principal du package Emailing CRM.");
                // Add the Editor and Bulk mail senders in the Crm Tools
                pagesDB.RecoverPage(StStr.PageEMailingTemplateEditor,StStr.CommonPortalID,StStr.PageEMailingMenuID,0);
                pagesDB.UpdatePageName(StStr.PageEMailingTemplateEditor,"EMail template editor");
                pagesDB.UpdatePageSettings(StStr.PageEMailingTemplateEditor,"PAGEMETADESCRIPTION_EN","Create or update email templates with a full html editor.");
                pagesDB.UpdatePageSettings(StStr.PageEMailingTemplateEditor,"PAGEMETADESCRIPTION_FR","Crée ou modifie les modèles d'email dans un editeur html.");
                moduleGuid              =   new Guid(AStr.CrmMailTemplateEditorGuid);
                // We must ensure that this module is available in common portal
                modDB.UpdateModuleDefinitions(moduleGuid,StStr.CommonPortalID,true);
                moduleDefId             =   modDB.GetModuleDefinitionByGuid(StStr.CommonPortalID,moduleGuid);
                modDB.AddModule(StStr.PageEMailingTemplateEditor,1,0,StStr.contentPane,"Template Editor",moduleDefId,0,
                                                        DBAdmin2,DBAllUser2,
                                                        DBAdmin2,DBAdmin2,
                                                        DBAdmin2,DBAdmin2,
                                                        DBAdmin2,false,false);
                // Add the Bulk Send tool
                pagesDB.RecoverPage(StStr.PageEMailingBulkSend,StStr.CommonPortalID,StStr.PageEMailingMenuID,0);
                pagesDB.UpdatePageName(StStr.PageEMailingBulkSend,"Bulk send emails");
                pagesDB.UpdatePageSettings(StStr.PageEMailingBulkSend,"PAGEMETADESCRIPTION_EN","Send emails from marketing lists.");
                pagesDB.UpdatePageSettings(StStr.PageEMailingBulkSend,"PAGEMETADESCRIPTION_FR","Envoi d'email à partir de listes marketing.");
                moduleGuid              =   new Guid(AStr.CrmBulkSendMailGuid);
                // We must ensure that this module is available in common portal
                modDB.UpdateModuleDefinitions(moduleGuid,StStr.CommonPortalID,true);
                moduleDefId             =   modDB.GetModuleDefinitionByGuid(StStr.CommonPortalID,moduleGuid);
                modDB.AddModule(StStr.PageEMailingBulkSend,1,0,StStr.contentPane,"Bulk send mails",moduleDefId,0,
                                                        DBAdmin2,DBAllUser2,
                                                        DBAdmin2,DBAdmin2,
                                                        DBAdmin2,DBAdmin2,
                                                        DBAdmin2,false,false);

            }
            return true;
        }

        /// <summary>
        /// To call on package install
        /// Add Crm SiteMap package menu options
        /// </summary>
        public bool Install(CRMWrapperBase wrapper,int portalID)
        {

            PagesDB pagesDB	        =	new PagesDB();
            ModulesDB modDB			=	new ModulesDB();

            string DBAllUser        =   Config.InternallAllUsersRole;
		    string DBAllUser2		=   Config.InternallAllUsersRole+";";

	        string DBAdmin          =   Config.InternalAdminRole;
            string DBAdmin2         =   Config.InternalAdminRole+";";

            //Move EMailing CRM Menu Page to common portal without parent page 
            RecoverMainEMailingMenu(wrapper,portalID);
            // Create a local portal page to receipt emails contact links
            int wPageId             =   pagesDB.AddPage(portalID, StStr.PageEMailingMenuID,"EMail Target Page",StStr.strDefaultSkin, 40,
                                                        3,0,0,0,0,0,DBAllUser);
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_EN","Page to use as a target for click link in mails.");
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_FR","Page destinée à être la cible du lien contenu dans les emails.");
            Guid moduleGuid         =   new Guid(AStr.CrmMailTargetPageGuid);
            // We must ensure that this module is available in this portal
            modDB.UpdateModuleDefinitions(moduleGuid,portalID,true);
            int moduleDefId         =   modDB.GetModuleDefinitionByGuid(portalID,moduleGuid);
            modDB.AddModule(wPageId,1,0,StStr.contentPane,"EMail target Page",moduleDefId, 0,
                                                    DBAdmin2, DBAllUser2,
                                                    DBAdmin2, DBAdmin2,
                                                    DBAdmin2, DBAdmin2,
                                                    DBAdmin2, false, false);
            // Add all the pages to import documents in sessions
            wPageId                 =   pagesDB.AddPage(portalID, StStr.PageEMailingMenuID,"Private Doc Feeder",StStr.strDefaultSkin, 50,
                                                        3,0,0,0,0,0,DBAllUser);
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_EN","Fill download session from folder.");
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_FR","Insérer des documents à télécharger a partir d'un répertoire.");
            moduleGuid                  =   new Guid(AStr.PrivateDocFeederGuid);
            moduleDefId                 =   modDB.GetModuleDefinitionByGuid(portalID,moduleGuid);
            modDB.AddModule(wPageId,1,0,StStr.contentPane,"Private document folder feeder",moduleDefId,0,
                                                    DBAdmin2,DBAllUser2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,false,false);
            wPageId                     =   pagesDB.AddPage(portalID, StStr.PageEMailingMenuID,"Private Doc Xml Feeder",StStr.strDefaultSkin, 60,
                                                        3,0,0,0,0,0,DBAllUser);
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_EN","Fill download session from xml.");
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_FR","Insérer des documents à télécharger a partir d'un fichier xml.");
            moduleGuid                  =   new Guid(AStr.PrivateDocXmlFeederGuid);
            moduleDefId                 =   modDB.GetModuleDefinitionByGuid(portalID,moduleGuid);
            modDB.AddModule(wPageId,1,0,StStr.contentPane,"Private document xml feeder",moduleDefId,0,
                                                    DBAdmin2,DBAllUser2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,false,false);
            wPageId                     =   pagesDB.AddPage(portalID, StStr.PageEMailingMenuID,"Private Doc Csv Feeder",StStr.strDefaultSkin, 70,
                                                        3,0,0,0,0,0,DBAllUser);
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_EN","Fill download session from csv.");
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_FR","Insérer des documents à télécharger a partir d'un fichier csv.");
            moduleGuid                  =   new Guid(AStr.PrivateDocCsvFeederGuid);
            moduleDefId                 =   modDB.GetModuleDefinitionByGuid(portalID,moduleGuid);
            modDB.AddModule(wPageId,1,0,StStr.contentPane,"Private document csv feeder",moduleDefId,0,
                                                    DBAdmin2,DBAllUser2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,false,false);
            wPageId                     =   pagesDB.AddPage(portalID, StStr.PageEMailingMenuID,"Private Download",StStr.strDefaultSkin, 80,
                                                        3,0,0,0,0,0,DBAllUser);
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_EN","Client Download page for private documents.");
            pagesDB.UpdatePageSettings(wPageId,"PAGEMETADESCRIPTION_FR","Page client de téléchargement de documents.");
            moduleGuid                  =   new Guid(AStr.PrivateDownLoadGuid);
            moduleDefId                 =   modDB.GetModuleDefinitionByGuid(portalID,moduleGuid);
            modDB.AddModule(wPageId,1,0,StStr.contentPane,"Download Private Documents",moduleDefId,0,
                                                    DBAdmin2,DBAllUser2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,DBAdmin2,
                                                    DBAdmin2,false,false);
            if ( InstallTraceSwitch.Sw.TraceVerbose )
                Trace.WriteLine("[V]InstallPackage Install END.",InstallTraceSwitch.Sw.Info);
            return true;
        }
    }
    
    #endregion Install class for Package

    #region InitClass for package

    /// <summary>
	/// This class is dedicated to CRMImage package initialization: Cf the InitApplication class from ADCRM.Core
    /// It needs a declaration in web config
    /// Readen on Application start web.config loads the assembly containing thois class and instanciate the object in application object
    /// This allows installing various handlers and to complete the portal global settings with e-commerce specific settings
    /// </summary>
    public class CRMEMailingInit
    {
        // Insert the DynEntities in the static dictionnary to allow easy dynamic creation
        static CRMEMailingInit()
        {
            if ( CRMDynTraceSwitch.Sw.TraceVerbose )
                Trace.WriteLine( "[V]CRMEMailingInit static Ctor BEG.",CRMDynTraceSwitch.Sw.Info);
            Assembly asm            =   Assembly.GetExecutingAssembly();
            foreach (Type type in asm.GetTypes()) 
            {
              if ( type.IsSubclassOf(typeof(DynEntity)) && type.IsClass ) 
              {
                Type ntp            =   type.GetNestedType("Ftor",BindingFlags.NonPublic);
                bool isMiddle       =   (ntp == null );
                if ( CRMDynTraceSwitch.Sw.TraceVerbose )
                    Trace.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CRMEMailingInit static Ctor found Name: {0}, Intermediary class: {1}.",type.Name, isMiddle),CRMDynTraceSwitch.Sw.Info);
                if ( isMiddle ) // no need to instanciate
                    continue;
                ConstructorInfo ci  =   type.GetConstructor(Type.EmptyTypes);
                ci.Invoke(new Object[] { }) ; // just to force static member instanciation
                if ( CRMDynTraceSwitch.Sw.TraceVerbose )
                    Trace.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]DynEntity static Ctor after invoke Name: {0}.",type.Name),CRMDynTraceSwitch.Sw.Info);
              }
            }
            if ( CRMDynTraceSwitch.Sw.TraceVerbose )
                Trace.WriteLine( "[V]CRMEMailingInit static Ctor END.",CRMDynTraceSwitch.Sw.Info);
        
        }


        public CRMEMailingInit()
        {
            if (CRMEMailingTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CRMEMailingInit Ctor called at: {0}", DateTime.UtcNow.ToLongTimeString()), CRMEMailingTrace.Sw.Info);
            PortalSettingsDictionary.PortalSettingsCreatedEvent +=  new PortalSettingsCreatedEventHandler(PortalSettings_PortalSettingsCreatedEvent);
        }


        private void PortalSettings_PortalSettingsCreatedEvent(PortalSettingsCreatedEventArgs e)
        {
            if (CRMEMailingTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CRMEMailingInit PortalSettings_PortalSettingsCreatedEvent BEG BaseSettings.Count: {0}", e.BaseSettings.Count), CRMEMailingTrace.Sw.Info);
            SettingsDictionary _baseSettings        =   e.BaseSettings;
            try
            {
                // We add the CrmImage Group to the SettingItemGroups.AllGroups static collection
                SettingItemGroup _Group             =   SettingItemGroup.CRM_EMAILING_SETTINGS;

                if (!SettingItemGroups.AllGroups.ContainsKey((int)_Group))
                    SettingItemGroups.AllGroups.Add((int)_Group, AStr.CRMEMailingSettings);

                #region Global path for image files
                int _groupOrderBase                 =   (int)_Group;

                // Album Path Setting
                SettingItem UseExternalSmtpServer   =   new SettingItem(AStr.UseExternalSmtpServer, new BooleanDataType());
                UseExternalSmtpServer.Required      =   false;
                UseExternalSmtpServer.Value         =   StStr.strFalse;
                UseExternalSmtpServer.Group         =   _Group;
                UseExternalSmtpServer.Order         =   _groupOrderBase;
                UseExternalSmtpServer.ResourceFile  =   AStr.ResourceFile;
                UseExternalSmtpServer.ResourceItem  =   AStr.ResourceItem;
                UseExternalSmtpServer.EnglishName   =   "Use External Smtp Server";
                UseExternalSmtpServer.Description   =   "When checked uses an external Smt server to send mails.";
                _baseSettings.AddSetting(UseExternalSmtpServer);

                SettingItem SmtpServer              =   new SettingItem(AStr.SmtpServer, new StringDataType());
                SmtpServer.Required                 =   false;
                //SmtpServer.Value                    =   StStr.strFalse;
                SmtpServer.Group                    =   _Group;
                SmtpServer.Order                    =   _groupOrderBase+1;
                SmtpServer.ResourceFile             =   AStr.ResourceFile;
                SmtpServer.ResourceItem             =   AStr.ResourceItem;
                SmtpServer.EnglishName              =   "Smtp Server";
                SmtpServer.Description              =   "Smt server to use to send mails.";
                _baseSettings.AddSetting(SmtpServer);

                // Iframe Page Menu ID
                // Defaults to Image Menu but should have been changed in install for the created iFrame page
                SettingItem SmtpServerPort          =   new SettingItem(AStr.SmtpServerPort, new IntegerDataType());
                SmtpServerPort.Required             =   false;
                SmtpServerPort.Value                =   StStr.PageImageManMenuID.ToString(); 
                SmtpServerPort.Group                =   _Group;
                SmtpServerPort.Order                =   _groupOrderBase+2;
                SmtpServerPort.ResourceFile         =   AStr.ResourceFile;
                SmtpServerPort.ResourceItem         =   AStr.ResourceItem;
                SmtpServerPort.EnglishName          =   "Smtp Server Port";
                SmtpServerPort.Description          =   "Smtp Server Port  to use.";
                _baseSettings.AddSetting(SmtpServerPort);
                #endregion

            }
            catch (Exception ex)
            {
                if (CRMEMailingTrace.Sw.TraceError)
                    Trace.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]CRMEMailingInit PortalSettings_PortalSettingsCreatedEvent ex:{0}", ex), CRMEMailingTrace.Sw.Info);
                throw;
            }
            if (CRMEMailingTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]CRMEMailingInit PortalSettings_PortalSettingsCreatedEvent END ok Count: {0}", e.BaseSettings.Count), CRMEMailingTrace.Sw.Info);
        }

    #endregion InitClass for package
    }
}
