﻿/* $RCSFile: PortalEmailingModule.cs $
 * Apache license v2 2011 by Christian Surieux
 * $Source: ?/AddonNice.Emailing.Settings/UI/WebControls/PortalEmailingModule.cs $
 * $log$
 * Revision 23 2010/10/01 11:24:59  christian.surieux
 *   Fixing temporary version on 1/10/2010
 * Revision 10 2010/08/09 10:13:18  christian.surieux
 *   Added Log comment to all cs files
 */

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Globalization;


using AddonNice.Helpers;
using AddonNice.UI.WebControls;
using AddonNice.Diagnostics;
using AddonNice.CRMEMailing;
using AddonNice.Settings.CRMEMailing;
using AddonNice.CRMWrapper;
using AddonNice.CRMWrapper.CRMEMailing;

using Trace2    =   System.Diagnostics.Trace;
using AddonNice.Configuration.Settings;
using AddonNice.Configuration;
using AddonNice.UI.DataTypes;
using System.Xml;
using System.Xml.Xsl;
using System.Web;
using AddonNice.Core;
using AddonNice.Settings;
using System.Data;
using AddonNice.Authentication;
using Telerik.Web.UI;

namespace AddonNice.CRMEMailing.Settings.UI.WebControls
{
 	#region Local TraceSwitch 
	
    public class PrivateDocFeederTrace : ADNTraceSwitch
	{
		static public ADNTraceSwitch	Sw	            =	new ADNTraceSwitch("PrivateDocFeederTrace","Special Trace Switch for PrivateDocFeeder");
	}
	
    public class CrmBulkSendTrace : ADNTraceSwitch
	{
		static public ADNTraceSwitch	Sw	            =	new ADNTraceSwitch("CrmBulkSendTrace","Special Trace Switch for CrmBulkSend");
	}
	
    #endregion Local TraceSwitch 

    public class PortalEmailingModule : PortalCRMModuleControl
    {

		#region license

        private XRMEMailing _ImM;
        
        /// <summary>
        /// Check for ImageManager and AddonNice license
        /// </summary>
        protected override bool CheckLicense
        {
            get
            {
                if (_ImM == null)
                    _ImM = new XRMEMailing();
                if (_ImM != null)
                    return ( _ImM.License.IsValid && base.CheckLicense ) ;
                return false;
            }
        }

        /// Check for ImageManager AddonNice license demo mode
        public override bool IsDemo
        {
            get
            {
                if (_ImM == null)
                    _ImM = new XRMEMailing();
                if (_ImM != null)
                    return ( _ImM.DemoMode && base.IsDemo );
                return true;
            }
        #endregion license
        }
        
        #region module properties override

        public override string Category
		{
			get
			{
                return AStr.EmailingModuleCat;
			}
		}

		public override string CategoryName
		{
			get
			{
                return LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,AStr.EMAIL_MOD_CAT,AStr.EmailingModuleCat);
			}
		}

        #endregion module properties override
    }


    public class PortalEmailingProgressModule : PortalCrmProgressModuleControl
    {

        #region license

        private XRMEMailing _ImM;

        /// <summary>
        /// Check for ImageManager and AddonNice license
        /// </summary>
        protected override bool CheckLicense
        {
            get
            {
                if ( _ImM == null )
                    _ImM = new XRMEMailing();
                if ( _ImM != null )
                    return ( _ImM.License.IsValid && base.CheckLicense );
                return false;
            }
        }

        /// Check for ImageManager AddonNice license demo mode
        public override bool IsDemo
        {
            get
            {
                if ( _ImM == null )
                    _ImM = new XRMEMailing();
                if ( _ImM != null )
                    return ( _ImM.DemoMode && base.IsDemo );
                return true;
            }
        }
        #endregion license

        #region module properties override

        public override string Category
		{
			get
			{
                return AStr.EmailingModuleCat;
			}
		}

		public override string CategoryName
		{
			get
			{
                return LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,AStr.EMAIL_MOD_CAT,AStr.EmailingModuleCat);
			}
		}

        #endregion module properties override

    }


    public class BulkSendModule : PortalEmailingProgressModule
    {
        protected int MaxErrors                                       =   1;
        protected int ErrCount                                        =   0;

        #region properties

        protected bool forceOwner
        {
            get
            {
                return Settings[AStr.Assign2Owner].ToBool();
            }
        }


        protected ADNDynSystemUser mailOwner
        {
            get
            {
                if ( forceOwner )
                {
                    return DynEntity.GetFromCache<ADNDynSystemUser>(CrmStr.systemuser,Wrapper,ResetWrapper,Settings[AStr.TemplateOwner].ToGuid());
                }
                return CurrentUser.GetImpersonateUser(Wrapper);
            }
        }

        private AddonNiceUser _CurrentUser = null;
        public AddonNiceUser CurrentUser
        {
            get
            {
                if (_CurrentUser == null)
                {
                    if ( CrmBulkSendTrace.Sw.TraceVerbose )
                        Trace2.WriteLine("[V]BulkSendModule CurrentUser GET BEG.",CrmBulkSendTrace.Sw.Info);
                    try
                    {
                        _CurrentUser    =   new AddonNiceUser();
                    }
                    catch (Exception ex)
                    {
                        if ( CrmBulkSendTrace.Sw.TraceError )
                            Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]BulkSendModule CurrentUser GET ex: {0}", ex),CrmBulkSendTrace.Sw.Info);
                        throw;
                    }
                    if ( CrmBulkSendTrace.Sw.TraceVerbose )
                        Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule CurrentUser Get Key: [{0}],userName: [{1}]",
                             _CurrentUser.ProviderUserKey, _CurrentUser.UserName),CrmBulkSendTrace.Sw.Info);
                }
                if (_CurrentUser == null)
                {
                    const string msg    =   "[E]BulkSendModule CurrentUser is NULL";
                    if ( CrmBulkSendTrace.Sw.TraceError )
                        Trace2.WriteLine(msg,CrmBulkSendTrace.Sw.Info);
                    throw new Exception(msg);
                }
                return _CurrentUser;
            }
        }

        protected List<Guid> _LstActivities           =   null;
        protected List<Guid> LstActivities
        {
            get
            {
                if ( _LstActivities != null )
                    return _LstActivities;
                object ob                   =   ViewState["ListActivities"];
                if ( ob == null )
                {
                    _LstActivities          =   new List<Guid>();
                    ViewState["ListActivities"] =   _LstActivities;
                }
                else  _LstActivities        =   (List<Guid>)ob;
                return _LstActivities;
            }
            set
            {
                ViewState["ListActivities"] =   value;
                _LstActivities              =   value;
            }
        }

        protected List<Guid> _LstJobs                 =   null;
        protected List<Guid> LstJobs
        {
            get
            {
                if ( _LstJobs != null )
                    return _LstJobs;
                object ob                   =   ViewState["ListJob"];
                if ( ob == null )
                {
                    _LstJobs                =   new List<Guid>();
                    ViewState["ListJob"]    =   _LstJobs;
                }
                else  _LstJobs              =   (List<Guid>)ob;
                return _LstJobs;
            }
            set
            {
                ViewState["ListJob"]        =   value;
                _LstJobs                    =   value;
            }
        }

        #endregion properties

        protected SortedList<string,DataTable> TbTemplatesDic     =   new SortedList<string,DataTable>();
        protected DataTable GetMailTemplates(string entName,int langCode)
        {
			if (CrmBulkSendTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule GetMailTemplates BEG entName: {0}, langCode: {1}",entName,langCode),CrmBulkSendTrace.Sw.Info);
            if ( TbTemplatesDic.ContainsKey(entName) )
            {
                return TbTemplatesDic[entName];
            }
            DataTable Tb    =   ADNDynTemplate.ReadAllTb(Wrapper,ResetWrapper,entName,langCode,false);
            TbTemplatesDic.Add(entName,Tb);
			if (CrmBulkSendTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule GetMailTemplates END Count: {0}",(Tb != null) ? Tb.Rows.Count:0),CrmBulkSendTrace.Sw.Info);
            return  Tb;
        }

        /*
<?xml version="1.0" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text" indent="no"/>
<xsl:template match="/data">
<![CDATA[<P>Dear ]]>
<xsl:choose><xsl:when test="contact/fullname"><xsl:value-of select="contact/fullname" /></xsl:when><xsl:otherwise>Valued Customer</xsl:otherwise></xsl:choose>
<![CDATA[,</P> <P>We have not heard from you for a while. We wanted to check in and make sure that you are still having a great experience using our product(s). We have asked ]]>
<xsl:choose><xsl:when test="contact/ownerid/@name"><xsl:value-of select="contact/ownerid/@name" /></xsl:when><xsl:otherwise>a customer service representative</xsl:otherwise></xsl:choose>
<![CDATA[ to contact you next week to get your feedback on the product(s) you are currently using and to give you details about our upcoming products.</P> <P>Thank you.</P>]]>
</xsl:template></xsl:stylesheet>         
         */

        protected string xsltdoc="<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"><xsl:output method=\"text\" indent=\"no\"/><xsl:template match=\"template\"><xsl:for-each select=\"*\">"+
			"<xsl:apply-templates select=\".\" /></xsl:for-each></xsl:template><xsl:template match=\"text\"><xsl:value-of select=\".\" /></xsl:template>"+
            "<xsl:template match=\"slugs\"><![CDATA[<span class=\"dataslug\"><span style=\"display:none;\" id=\"DataSlug\">{!]]><xsl:for-each select=\"slug\"><xsl:value-of select=\"entity\" />:<xsl:value-of select=\"attribute\" />;</xsl:for-each><xsl:value-of select=\"default\" />"+
            "<![CDATA[}</span>{!]]><xsl:for-each select=\"slug\"><xsl:value-of select=\"entity/@displayname\" />:<xsl:value-of select=\"attribute/@displayname\" />;</xsl:for-each><xsl:value-of select=\"default\" /><![CDATA[}</span>]]></xsl:template></xsl:stylesheet>";

        protected string TransformXml(string xmlbody,string templatetypecode)
        {
            XmlDocument doc				        =   new XmlDocument();
			doc.LoadXml(xmlbody);

			XslCompiledTransform xslt			=   new XslCompiledTransform();

			// build parameter list to pass to stylesheet
			XsltArgumentList xslArg		        =   new XsltArgumentList();
            XmlDocument xdoc                    =   new XmlDocument();
            xdoc.LoadXml(xsltdoc.Replace("slugs",templatetypecode));
			//Load the stylesheet
			xslt.Load(xdoc,null,null);
			//Transform the data
            StringBuilder sb                    =   new StringBuilder();
            StringWriter sw                     =   new StringWriter(sb);
            //MemoryStream ms                     =   new MemoryStream();
            xslt.Transform(doc.CreateNavigator(), xslArg,sw);
            //ms.Seek(0, SeekOrigin.Begin);
            /*
            XmlDocument doc2                    =   new XmlDocument();
            doc2.Load(ms);*/
            string result   =   sb.ToString();//doc2.OuterXml;
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule body: [{0}].",result),CrmBulkSendTrace.Sw.Info);
			return result;
        }

        public virtual void SetMessage(string txt)
        {}

        public string MakeTargetUrl(int TargetPage,string attr,bool ForExternal)
		{
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule MakeTargetUrl BEG TargetPage: {0}.",TargetPage),CrmBulkSendTrace.Sw.Info);
            UriBuilder ub           =   new UriBuilder(HttpContext.Current.Request.Url);
            string targetUrl        =   HttpUrlBuilder.BuildUrlWithCustomAttributes(TargetPage,attr);
            string defDom           =   Settings[AStr.TargetDomain].ToString().Trim();
            targetUrl               =   targetUrl.Replace(ub.Host,defDom);
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule MakeTargetUrl TargetPage: {0}.",TargetPage),CrmBulkSendTrace.Sw.Info);
            if ( ForExternal )
            {
                string port2Replace =   string.Format(":{0}",Config.AdminPortalPort);
                if ( targetUrl.IndexOf(port2Replace) > -1 )
                {
                    if ( Config.PortalPort != 80 )
                        targetUrl    =  targetUrl.Replace(port2Replace,string.Format(":{0}",Config.PortalPort));
                    else
                        targetUrl   =   targetUrl.Replace(port2Replace,string.Empty);
                }
            }
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule MakeTargetUrl END targetUrl: {0}.",targetUrl),CrmBulkSendTrace.Sw.Info);
            return  targetUrl;
        }

        /// <summary>
        /// Update the mail body with keywords and options
        /// For normal mails in a campaign replace ADN_TARGETPAGE with input target page and all the input parameters necessary to identify the sender, email an campaign if any
        /// For download sessions use always the same target page but add parameters to allow finding requested documents (note that target page could request a password)
        /// Send the mail if settings set for this (default=send)
        /// Create the mail in Crm if flag request it
        /// </summary>
        public string UpdateMailContent(Guid SessionId,ADNDynEmail mail,string MemberTypename,DynEntity dynCust,Guid CampaignId,int TargetPage,bool create,bool Unsubscribe,string UnsubscribeText,bool MailOpen,string ClickUrlText)
        {
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent BEG create: {0}, CampaignId: {1}, TargetPage: {2}, MemberTypename: {3}, dynCust.PrimaryFieldStringValue: {4}, isSession: {5}.",
                                                                                                    new object[]{create,CampaignId,TargetPage,MemberTypename,dynCust.PrimaryFieldStringValue,SessionId}),CrmBulkSendTrace.Sw.Info);
            // If not created we need a mailid so do it
            try
            {
                if ( create )
                    mail.Create();
            }
            catch(Exception ex)
            {
                if ( CrmBulkSendTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent create: {0}, ex: {1}.",create,ex),CrmBulkSendTrace.Sw.Info);
                ErrCount++;
                if ( ErrCount >= MaxErrors )
                {
                    SetMessage(string.Format("MaxError: {0}",ErrCount));
                    throw;
                }
            }
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent mailid: {0}, statuscode: {1}.",mail.activityid,mail.statuscode),CrmBulkSendTrace.Sw.Info);
            string body                             =   mail.description;
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent dynCust: {0}, dicProperty.Count: {1}.",dynCust,dynCust.dicProperty.Count),CrmBulkSendTrace.Sw.Info);
            string attr                             =   (CampaignId == Guid.Empty ) ?
                                                            string.Format("e={0}&id={1}&m={3}",new object[]{MemberTypename,dynCust.PrimaryKeyGuidValue,mail.activityid})
                                                            :
                                                            string.Format("e={0}&id={1}&c={2}&m={3}",new object[]{MemberTypename,dynCust.PrimaryKeyGuidValue,CampaignId,mail.activityid});
            // replace standard elements
            if ( body.IndexOf("ADN_TARGETPAGE") > -1 )
            {
                string targetUrl                    =   string.Format("{0}?{1}",ClickUrlText,attr);
                mail.description                    =   body.Replace("ADN_TARGETPAGE",targetUrl);
                if ( CrmBulkSendTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule FinishBtn_Click updated mail.description: [{0}].",mail.description),CrmBulkSendTrace.Sw.Info);
            }
            // Insert Unsubscribe
            if ( Unsubscribe)
            {
                if ( string.IsNullOrEmpty(mail.trackingtoken) )
                {
                    mail.GetTrackingToken();
                }
//mailto:christian.surieux@addonnice.com?subject=UNSUBSCRIBE:%20Nous%20vous%20contactons%20pour%20vous%20présenter%20notre%20offre%20de%20services%20applicatifs%20hébergés%20pour%20la%20gestion%20de%20vos%20forces%20de%20vente%20ADNCRM:0001101
                string unsubtxt                     =   UnsubscribeText;
                if ( string.IsNullOrEmpty(unsubtxt) )
                    unsubtxt                        =   LocalizeHelper.Localize(AStr.ResourceFile,AStr.ResourceItem,"TEMPLATE_TXTUNSUBSCRIBE","Click here to unsubscribe");
                mail.description                    =   string.Format("{0}<a href='mailto:{1}?subject=UNSUBSCRIBE:{2}' >{3}</a>",
                    mail.description,mailOwner.internalemailaddress,Server.HtmlEncode(mail.subject),Server.HtmlEncode(unsubtxt));
            }

            // Tracking mail open insert onePixel tracer: t_mailGuid.png
            if ( MailOpen )
            {
                if ( Settings[AStr.OpenHandlerType].Value == "Image" )
                     mail.description                    =   string.Format("{0}<img src='/images/t_{1}.png' alt='logo' />",mail.description,mail.activityid);
                else 
                {
                    string targetUrl                    =   ClickUrlText;
                    Uri targetUri                       =   new Uri(targetUrl);
                    string imgHdlr                      =   Settings[AStr.ImgHandlerName].ToString().Trim();
                    string url                          =   string.Empty;
                    if ( imgHdlr.IndexOf("http") == 0 )
                            url                         =   imgHdlr;
                    else    url                         =   string.Format("{0}://{1}/{2}?{3}",new object[]{targetUri.Scheme,targetUri.Host,imgHdlr,attr});
                    mail.description                    =   string.Format("{0}<img src='{1}' alt='' />",mail.description,url);
                }
                if ( CrmBulkSendTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent after MailOpen mail.description: [{0}].",mail.description),CrmBulkSendTrace.Sw.Info);
            }
            try
            {
                mail.Update();
            }
            catch(Exception ex)
            {
                if ( CrmBulkSendTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent create: {0}, ex: {1}.",create,ex),CrmBulkSendTrace.Sw.Info);
                ErrCount++;
                if ( ErrCount >= MaxErrors )
                {
                    SetMessage(string.Format("MaxError: {0}",ErrCount));
                    throw;
                }
                return "Err: "+mail.subject; // do not send if error
            }
            bool prepareOnly        =   !Settings[AStr.SendMails].ToBool();
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent PrepareOnly: {0}, activityid: {1}.",prepareOnly,mail.activityid),CrmBulkSendTrace.Sw.Info);
            if ( prepareOnly )
                return mail.subject;
            string newSubject                       =   mail.Send(false);
            if ( CrmBulkSendTrace.Sw.TraceVerbose )
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule UpdateMailContent END sent Subject: {0}.",newSubject),CrmBulkSendTrace.Sw.Info);
            return newSubject;
        }


        protected virtual int GetLangCode()
        {
            return 0;
        }
        /// <summary>
        /// Fill the templates combo 
        /// </summary>
        protected void MarketingLists_ItemDataBound(object sender,GridItemEventArgs e)
        {
            if ( !( e.Item is GridDataItem ) )
                return;
            GridDataItem Item                       =   e.Item as GridDataItem;
			if (CrmBulkSendTrace.Sw.TraceVerbose)
                Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]BulkSendModule MarketingLists_ItemDataBound BEG IsInEditMode: {0}",Item.IsInEditMode),CrmBulkSendTrace.Sw.Info);
            Guid listid                             =   new Guid(Item.GetDataKeyValue("listid").ToString());
            ADNDynList lst                          =   DynEntity.GetFromCache<ADNDynList>(CrmStr.list,Wrapper,ResetWrapper,listid);
            if ( !lst.Retrieved )
            {
                const string msg                    =   "[E]BulkSendModule MarketingLists_ItemDataBound list not found.";
                if (CrmBulkSendTrace.Sw.TraceError)
                    Trace2.WriteLine(msg,CrmBulkSendTrace.Sw.Info);
                throw new Exception(msg);
            }
            RadComboBox CbTemplate                  =   ((RadComboBox)Item.FindControl("CbTemplate"));
            string entName                          =   lst.MemberTypename;
            if ( CbTemplate != null )
            {
                int langCode                        =   GetLangCode();
                CbTemplate.DataTextField            =   CrmStr.title;
                CbTemplate.DataValueField           =   CrmStr.templateid;
                CbTemplate.DataSource               =   GetMailTemplates(entName,langCode);
                CbTemplate.DataBind();
            }
            int typCde                              =   (int)((DataRowView)Item.DataItem)[CrmStr.membertype];
            Item[CrmStr.membertype].Text            =   entName;
            if (CrmBulkSendTrace.Sw.TraceVerbose)
                Trace2.WriteLine("[V]BulkSendModule MarketingLists_ItemDataBound END.",CrmBulkSendTrace.Sw.Info);

        }

    }

 	/// <summary>
    /// Initializer class for PrivateDocCsvFeeder 
    /// </summary>
    public class PrivateDocFeeder0SI : PortalCRMModuleControlSI
    {
        public PrivateDocFeeder0SI(ModuleSettingsDictionary bSettings,ISettingsOwner pm) : base()
        {
            if ( PrivateDocFeederTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]PrivateDocFeeder0SI Ctor BEG.",PrivateDocFeederTrace.Sw.Info);
            //InitSettings(bSettings,pm);
            if ( PrivateDocFeederTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]PrivateDocFeeder0SI Ctor END.",PrivateDocFeederTrace.Sw.Info);
        }
 
        public override void InitSettings(ModuleSettingsDictionary bSettings,ISettingsOwner pm)
        {             
            if ( PrivateDocFeederTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]PrivateDocFeeder0SI InitSettings BEG.",PrivateDocFeederTrace.Sw.Info);
            
            base.InitSettings(bSettings,pm);

            SettingItemSpecial DefaultFileExt =   new SettingItemSpecial(AStr.DefaultFileExt,new StringDataType());
            DefaultFileExt.Required           =   false;
            DefaultFileExt.ResourceFile       =   AStr.ResourceFile;
            DefaultFileExt.ResourceItem       =   AStr.ResourceItem;
            DefaultFileExt.Value              =   "txt";
            DefaultFileExt.Order              =   10;
            DefaultFileExt.ExtendedGroupName  =   pm.ExtendedResourcesGroup;
            DefaultFileExt.EnglishName        =   "Default file extensions";
            DefaultFileExt.Description        =   "File extension to use loading documents";
            bSettings.AddSetting(DefaultFileExt);
           
            if ( PrivateDocFeederTrace.Sw.TraceVerbose )
                Trace2.WriteLine( "[V]PrivateDocFeeder0SI InitSettings END.",PrivateDocFeederTrace.Sw.Info);
        }
 
    }

    public class PrivateDocFeeder0 : PortalEmailingProgressModule
    {
        protected XrmEMailingDB  mailDB   =   new XrmEMailingDB();

        public bool LoadFile(ADNDynPrivateDocument priv,string path2,string title,string filename,bool created)
        {
            if (PrivateDocFeederTrace.Sw.TraceVerbose)
                Trace2.WriteLine(  string.Format(CultureInfo.InvariantCulture,"[V]PrivateDocFeeder LoadFile BEG path2: {0}, titlel: {1}.",path2,title),PrivateDocFeederTrace.Sw.Info);
            Guid pId                    =   priv.PrimaryKeyGuidValue;
            FileStream fs               =   null;
            try
            {
                FileInfo fi             =   new FileInfo(path2);
                Byte[] byteAr2          =   new Byte[fi.Length];
                if ( PrivateDocFeederTrace.Sw.TraceVerbose )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[V]PrivateDocFeeder LoadFile fi.Length: {0}", fi.Length),PrivateDocFeederTrace.Sw.Info);
                fs                      =   new FileStream(path2, FileMode.Open, FileAccess.Read, FileShare.Read);
                fs.Read(byteAr2, 0, (int)fi.Length);
                int fileid              =   priv.adnexternalid;
                if (  fileid == 0 )
                        fileid          =   mailDB.AddPrivateDocument(pId,title,string.Empty,(int)fi.Length,filename,path2,DateTime.UtcNow,byteAr2);
                else    mailDB.UpdatePrivateDocument(fileid,pId,filename,string.Empty,(int)fi.Length,filename,path2,byteAr2);
                if (PrivateDocFeederTrace.Sw.TraceVerbose)
                    Trace2.WriteLine(  string.Format(CultureInfo.InvariantCulture,"[V]PrivateDocFeeder LoadFile fileid: {0}.",fileid),PrivateDocFeederTrace.Sw.Info);
                // bring the fileid to crm
                if ( created )
                {
                    priv.adnexternalid  =   fileid;
                    priv.Update();
                    if (PrivateDocFeederTrace.Sw.TraceVerbose)
                        Trace2.WriteLine(  string.Format(CultureInfo.InvariantCulture,"[V]PrivateDocFeeder LoadFile updated with fileid: {0}.",fileid),PrivateDocFeederTrace.Sw.Info);
                }
            }
            catch (Exception ex)
            {
                if ( PrivateDocFeederTrace.Sw.TraceError )
                    Trace2.WriteLine( string.Format(CultureInfo.InvariantCulture,"[E]PrivateDocFeeder LoadFile path2: {0}, ex: {1}", path2, ex),PrivateDocFeederTrace.Sw.Info);
                return false;
            }
            finally
            {
                if ( fs != null )
                    fs.Close();
            }
            return true;
        }

    }
}
